/* Serial test program for MyIRC/irman - 2006 Mike Bourgeous */

#define _XOPEN_SOURCE	600
#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/poll.h>

int main()
{
	struct termios term_attr;
	struct pollfd term_poll = { 0, POLLIN, 0};
	int term = 0;
	unsigned char command = 0;
	unsigned char result[6] = {0,0,0,0,0,0};
	unsigned char ok[2] = {0,0};
	int poll_timeout = 10000;
	int done = 0;
	int i, j;
	char *port;

	port = getenv("MYPORT");
	
	if(!(term = open(port ? port : "/dev/ttyS0", O_NOCTTY | O_RDWR))) {
		perror("Unable to open serial port");
		return -1;
	}

	term_poll.fd = term;

	tcgetattr(term, &term_attr);

	/* printf("Old c_cflag: %08x\n", term_attr.c_cflag); */
	term_attr.c_cflag = B9600 | CS8 | CLOCAL | CREAD | HUPCL;
	term_attr.c_oflag = 0;
	term_attr.c_lflag = 0;
	term_attr.c_iflag = 0;
	cfsetispeed(&term_attr, 9600);
	cfsetospeed(&term_attr, 9600);
	term_attr.c_cc[VMIN] = 1;
	term_attr.c_cc[VTIME] = 0;

	/* printf("New c_cflag: %08x\n", term_attr.c_cflag); */
	
	if(tcsetattr(term, TCSAFLUSH, &term_attr)) {
		perror("Unable to set serial port parameters");
		close(term);
		return -1;
	}

	/* Attempt to initialize IR receiver */
	/* Read 'X' */
	if(read(term, ok, 1) != 1) {
		perror("Unable to initialize receiver");
		close(term);
		return -1;
	}
	printf("x: %c (%02X)\n", ok[0], ok[0]);

	if(ok[0] != 'X') {
		printf("No 'X' received - is this a MyHD receiver?\n");
	}
	
	/* Write 'IR' */
	command = 'I';
	if(write(term, &command, 1) != 1) {
		perror("Unable to initialize receiver");
		close(term);
		return -1;
	}
	
	command = 'R';
	if(write(term, &command, 1) != 1) {
		perror("Unable to initialize receiver");
		close(term);
		return -1;
	}

	/* Read 'OK' */
	if(read(term, ok, 2) != 2 || ok[0] != 'O' || ok[1] != 'K') {
		if(errno) {
			perror("Read failed");
		}
		fprintf(stderr, "No MyHD IR receiver found\n");
		close(term);
		return -1;
	}

	printf("ok: %c%c (%02X %02X)\n", ok[0], ok[1], ok[0], ok[1]);

	while(done == 0) {
		switch(poll(&term_poll, 1, poll_timeout)) {
			case 0:
				/* 0 indicates timeout was reached */
				printf("No data from IR receiver for ten seconds\n");
				done = 1;
				break;
			case -1:
				/* -1 indicates error on poll */
				perror("Error waiting for IR receiver data");
				done = 2;
				break;
			default:
				/* Otherwise, poll succeeded */
				if(read(term, result, 6) > 0) {
					printf("ir: %02X %02X %02X %02X %02X %02X - ",
						result[0], result[1], result[2], result[3], result[4], result[5]);
					for(j = 0; j < 6; j++) {
						for(i = 7; i >= 0; i--) {
							printf("%1x", (result[j] & (1 << i)) ? 1 : 0);
						}
						printf(" ");
					}
					printf("\n");
				}
				break;
		}
		if(term_poll.fd != term) {
			term_poll.fd = term;
		}
		if(term_poll.events != POLLIN) {
			term_poll.events = POLLIN;
		}
	}

	close(term);

	return -(done - 1);
}

